import { TextInput, DialogService, LoadingService } from "@zeedhi/common";
import { Metadata, Http, DateHelper } from '@zeedhi/core';
import { TekGrid } from "@zeedhi/teknisa-components-common";
import { CustomAlertService } from '@/Services/CustomAlertService';
import { PeriodsService } from "./PeriodsService";

export class Utilities {

    public static getNameDynamic(name: string) {
        const date = new Date();
        return name + date.getTime();
    }

    public static lightColorHexadecimalValidity(color: String) {
        const hex = color.replace('#', '');

        const r = parseInt(hex.substr(0, 2), 2);
        const g = parseInt(hex.substr(2, 2), 2);
        const b = parseInt(hex.substr(4, 2), 2);

        const lum = (r * 299 + g * 587 + b * 114) / 1000;
    }

    public static roundFloat(num: number, decimalPlaces: number = 2) {
        const tenPower = 10 ** decimalPlaces;
        return Math.round((num + Number.EPSILON) * tenPower) / tenPower;
    }

    public static fixBugHintOnfocus(component: TextInput, element: HTMLElement | undefined, count = 1) {
        setTimeout(() => {
            if (element) {
                const messages = element.querySelector('.v-messages__message');
                if (messages) {
                    if (component.hint) {
                        messages.innerHTML = component.hint;
                    }

                } else if (count < 5) {
                    count += 1;
                    Utilities.fixBugHintOnfocus(component, element, count);
                }
            }

        });
    }

    public static async genericBeforeSaveTekGrid(params: any, route: string, tekGridName: string) {
        params.event.preventDefault();
        try {
            LoadingService.show();
            await Http.post(route, {
                "requestType": "DataSet",
                "dataset": params.component.getEditedRows()
            });
            LoadingService.hide();
            CustomAlertService.saveSuccessAlert();
            params.component.cancelEditedRows();
            Metadata.getInstance<TekGrid>(tekGridName).datasource.get();
        } catch (e) {
            LoadingService.hide();
            CustomAlertService.errorAlert(e);
        }
    }

    // Before Delete Methods

    public static deleteRoute: string;
    public static deleteTekGridName: string;
    public static deleteRequestType: string;

    public static genericBeforeDeleteTekGrid(params: any, route: string, tekGridName: string, requestType: string = 'DataSet') {
        params.event.preventDefault();
        this.deleteRoute = route;
        this.deleteTekGridName = tekGridName;
        this.deleteRequestType = requestType;
        DialogService.show({
          name: 'delete-dialog_' + tekGridName,
          type: 'warning',
          title: ' ',
          text: 'Deseja remover o(s) registro(s) selecionado(s)?',
          buttons: [
            { name: 'não', component: 'ZdButton', label: 'NÃO', outline: true, events: { click: () => DialogService.hide(), }, },
            { name: 'sim', component: 'ZdButton', label: 'SIM', events: Utilities.deleteTekGrid, },
          ],
        });
    }

    private static deleteTekGrid: any = {
        click: () => Utilities.doDeleteTekGrid(),
    };

    private static async doDeleteTekGrid() {
        DialogService.hide();
        try {
            const grid = Metadata.getInstance<TekGrid>(this.deleteTekGridName);
            const requestTypeAttr = this.deleteRequestType === "DataSet" ? "dataset" : "row";
            LoadingService.show();
            await Http.post(Utilities.deleteRoute, {
                "requestType": this.deleteRequestType,
                [requestTypeAttr]: grid.selectedRows
            });
            LoadingService.hide();
            CustomAlertService.deleteSuccessAlert();
            grid.datasource.get();
            grid.selectedRows = [];
        } catch (e) {
            LoadingService.hide();
            CustomAlertService.errorAlert(e);
        }
        this.deleteRoute = '';
        this.deleteTekGridName = '';
    }

    public static async getLoggedUser() {
        const res = await Http.post("/getLoggedUser", {nrorg: null, nrorgPrincipal: null});
        return res.data;
    }

    public static async generatePeriodHelperOptions(period: any, helperValue: any) {
        const res = await Http.post("/periods");
        const data = res.data.data;
        if (period.helperOptions && period.helperOptions.length > 0) {
            period.helperOptions.forEach((option: any) => DateHelper.unregister(option.periodsCode));
        }
        data.forEach((option: any) => DateHelper.register(option.periodsCode, option.name, () => {
            const service = new PeriodsService();
            const periods = service.decodePeriod(option.periodsCode);
            return [periods.start, periods.end].map((date: any) => date.split('/').reverse().join('-')) as any;
        }));
        period.helperOptions = data.map((option: any) => option.periodsCode).filter((val: any) => val);
        period.helperValue = helperValue;
        period.updateHelperHint();
    }

    public static setPeriodHelperValue(period: any, value: any) {
        if (value) {
            if (typeof value === 'string') {
                const service = new PeriodsService();
                const values = service.decodePeriod(value);
                period.setDateValue(values.start + period.splitter + values.end);
                if (period.helperOptions?.includes(value)) {
                    period.helperValue = value;
                }
            } else if (value.start && value.end) {
                period.setDateValue(value.start + period.splitter + value.end);
            } else if (Array.isArray(value)) {
                period.setDateValue(value[0] + period.splitter + value[1]);
            } else {
                period.setDateValue(value);
            }
        } else {
            period.setDateValue("");
        }
    }
}
